import { TriggerTracer, SemanticInternalAttributes, apiClientManager, isRequestOptions, mergeRequestOptions, accessoryAttributes, flattenAttributes, taskContext, runtime, usage as usage$1, UnprocessableEntityError, logger, taskCatalog, TimezonesResult, defaultRetryOptions, calculateNextRetryDelay, defaultFetchRetryOptions, calculateResetAt, eventFilterMatches, stringifyIO, conditionallyImportPacket, parsePacket, createErrorTaskError } from '@systemfsoftware/trigger.dev_core/v3';
export { AbortTaskRunError, ApiError, AuthenticationError, BadRequestError, ConflictError, InternalServerError, NotFoundError, PermissionDeniedError, RateLimitError, UnprocessableEntityError, logger } from '@systemfsoftware/trigger.dev_core/v3';
import { trace, context, SpanStatusCode, SpanKind } from '@opentelemetry/api';
import { SEMATTRS_HTTP_STATUS_CODE, SEMATTRS_HTTP_METHOD, SEMATTRS_HTTP_URL, SEMATTRS_HTTP_HOST, SEMATTRS_HTTP_SCHEME, SEMATTRS_HTTP_RESPONSE_CONTENT_LENGTH, SEMATTRS_MESSAGING_OPERATION, SEMATTRS_MESSAGING_SYSTEM, SEMATTRS_MESSAGING_DESTINATION } from '@opentelemetry/semantic-conventions';
import { AsyncLocalStorage } from 'node:async_hooks';
import { zodfetch } from '@systemfsoftware/trigger.dev_core/v3/zodfetch';

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name2 in all)
    __defProp(target, name2, { get: all[name2], enumerable: true });
};
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};

// package.json
var version = "2.4.4";

// src/v3/tracer.ts
var tracer = new TriggerTracer({
  name: "@systemfsoftware/trigger.dev_sdk",
  version
});

// src/v3/cache.ts
var _InMemoryCache = class _InMemoryCache {
  constructor() {
    __publicField(this, "_cache", /* @__PURE__ */ new Map());
  }
  get(key) {
    return this._cache.get(key);
  }
  set(key, value) {
    this._cache.set(key, value);
    return void 0;
  }
  delete(key) {
    this._cache.delete(key);
    return void 0;
  }
};
__name(_InMemoryCache, "InMemoryCache");
var InMemoryCache = _InMemoryCache;
function createCache(store) {
  return /* @__PURE__ */ __name(function cache(cacheKey, fn) {
    return tracer.startActiveSpan("cache", async (span) => {
      span.setAttribute("cache.key", cacheKey);
      span.setAttribute(SemanticInternalAttributes.STYLE_ICON, "device-sd-card");
      const cacheEntry = await store.get(cacheKey);
      if (cacheEntry) {
        span.updateName(`cache.hit ${cacheKey}`);
        return cacheEntry.value;
      }
      span.updateName(`cache.miss ${cacheKey}`);
      const value = await tracer.startActiveSpan("cache.getFreshValue", async (span2) => {
        return await fn();
      }, {
        attributes: {
          "cache.key": cacheKey,
          [SemanticInternalAttributes.STYLE_ICON]: "device-sd-card"
        }
      });
      await tracer.startActiveSpan("cache.set", async (span2) => {
        await store.set(cacheKey, {
          value,
          metadata: {
            createdTime: Date.now()
          }
        });
      }, {
        attributes: {
          "cache.key": cacheKey,
          [SemanticInternalAttributes.STYLE_ICON]: "device-sd-card"
        }
      });
      return value;
    });
  }, "cache");
}
__name(createCache, "createCache");
function onThrow(fn, options) {
  const opts = {
    ...defaultRetryOptions,
    ...options
  };
  return tracer.startActiveSpan(`retry.onThrow()`, async (span) => {
    let attempt = 1;
    while (attempt <= opts.maxAttempts) {
      const innerSpan = tracer.startSpan("retry.fn()", {
        attributes: {
          [SemanticInternalAttributes.STYLE_ICON]: "function",
          ...accessoryAttributes({
            items: [
              {
                text: `${attempt}/${opts.maxAttempts}`,
                variant: "normal"
              }
            ],
            style: "codepath"
          })
        }
      });
      const contextWithSpanSet = trace.setSpan(context.active(), innerSpan);
      try {
        const result = await context.with(contextWithSpanSet, async () => {
          return fn({
            attempt,
            maxAttempts: opts.maxAttempts
          });
        });
        innerSpan.end();
        return result;
      } catch (e) {
        if (e instanceof Error || typeof e === "string") {
          innerSpan.recordException(e);
        } else {
          innerSpan.recordException(String(e));
        }
        innerSpan.setStatus({
          code: SpanStatusCode.ERROR
        });
        if (e instanceof Error && e.name === "AbortTaskRunError") {
          innerSpan.end();
          throw e;
        }
        const nextRetryDelay = calculateNextRetryDelay(opts, attempt);
        if (!nextRetryDelay) {
          innerSpan.end();
          throw e;
        }
        innerSpan.setAttribute(SemanticInternalAttributes.RETRY_AT, new Date(Date.now() + nextRetryDelay).toISOString());
        innerSpan.setAttribute(SemanticInternalAttributes.RETRY_COUNT, attempt);
        innerSpan.setAttribute(SemanticInternalAttributes.RETRY_DELAY, `${nextRetryDelay}ms`);
        innerSpan.end();
        await runtime.waitForDuration(nextRetryDelay);
      } finally {
        attempt++;
      }
    }
    throw new Error("Max attempts reached");
  }, {
    attributes: {
      [SemanticInternalAttributes.STYLE_ICON]: "arrow-capsule"
    }
  });
}
__name(onThrow, "onThrow");
var normalizeUrlFromInput = /* @__PURE__ */ __name((input) => {
  if (typeof input === "string") {
    return new URL(input);
  }
  if (input instanceof URL) {
    return input;
  }
  return new URL(input.url);
}, "normalizeUrlFromInput");
var normalizeHttpMethod = /* @__PURE__ */ __name((input, init) => {
  if (typeof input === "string" || input instanceof URL) {
    return (init?.method || "GET").toUpperCase();
  }
  return (input.method ?? init?.method ?? "GET").toUpperCase();
}, "normalizeHttpMethod");
var fetchHttpHandlerStorage = new AsyncLocalStorage();
var fetchWithInterceptors = /* @__PURE__ */ __name(async (input, init) => {
  const handlers = fetchHttpHandlerStorage.getStore();
  if (handlers) {
    try {
      const { getResponse } = await import('msw');
      const request = new Request(input, init);
      const response = await getResponse(handlers, request);
      if (response) {
        return response;
      }
    } catch (e) {
      return fetch(input, init);
    }
  }
  return fetch(input, init);
}, "fetchWithInterceptors");
var _a;
var FetchErrorWithSpan = (_a = class extends Error {
  constructor(originalError, span) {
    super("Fetch error");
    __publicField(this, "originalError");
    __publicField(this, "span");
    this.originalError = originalError;
    this.span = span;
  }
}, __name(_a, "FetchErrorWithSpan"), _a);
var MAX_ATTEMPTS = 10;
async function retryFetch(input, init) {
  return tracer.startActiveSpan("retry.fetch()", async (span) => {
    let attempt = 1;
    while (true) {
      try {
        const abortController = new AbortController();
        const timeoutId = init?.timeoutInMs ? setTimeout(() => {
          abortController.abort();
        }, init?.timeoutInMs) : void 0;
        init?.signal?.addEventListener("abort", () => {
          abortController.abort();
        });
        const [response, span2] = await doFetchRequest(input, {
          ...init ?? {},
          signal: abortController.signal
        }, attempt);
        if (timeoutId) {
          clearTimeout(timeoutId);
        }
        if (response.ok) {
          span2.setAttributes(createFetchResponseAttributes(response));
          span2.end();
          return response;
        }
        const nextRetry = await calculateRetryDelayForResponse(resolveDefaults(init?.retry, "byStatus", defaultFetchRetryOptions.byStatus), response, attempt);
        if (!nextRetry) {
          span2.setAttributes(createFetchResponseAttributes(response));
          span2.end();
          return response;
        }
        if (attempt >= MAX_ATTEMPTS) {
          span2.setAttributes(createFetchResponseAttributes(response));
          span2.end();
          return response;
        }
        if (nextRetry.type === "delay") {
          span2.setAttribute(SemanticInternalAttributes.RETRY_AT, new Date(Date.now() + nextRetry.value).toISOString());
          span2.setAttribute(SemanticInternalAttributes.RETRY_COUNT, attempt);
          span2.setAttribute(SemanticInternalAttributes.RETRY_DELAY, `${nextRetry.value}ms`);
          span2.end();
          await runtime.waitForDuration(nextRetry.value);
        } else {
          const now = Date.now();
          const nextRetryDate = new Date(nextRetry.value);
          const isInFuture = nextRetryDate.getTime() > now;
          span2.setAttribute(SemanticInternalAttributes.RETRY_AT, new Date(nextRetry.value).toISOString());
          span2.setAttribute(SemanticInternalAttributes.RETRY_COUNT, attempt);
          if (isInFuture) {
            span2.setAttribute(SemanticInternalAttributes.RETRY_DELAY, `${nextRetry.value - now}ms`);
          }
          span2.end();
          await runtime.waitUntil(new Date(nextRetry.value));
        }
      } catch (e) {
        if (e instanceof FetchErrorWithSpan && e.originalError instanceof Error) {
          if (e.originalError.name === "AbortError") {
            const nextRetryDelay = calculateNextRetryDelay(resolveDefaults(init?.retry, "timeout", defaultFetchRetryOptions.timeout), attempt);
            if (!nextRetryDelay) {
              e.span.end();
              throw e;
            }
            if (attempt >= MAX_ATTEMPTS) {
              e.span.end();
              throw e;
            }
            e.span.setAttribute(SemanticInternalAttributes.RETRY_AT, new Date(Date.now() + nextRetryDelay).toISOString());
            e.span.setAttribute(SemanticInternalAttributes.RETRY_COUNT, attempt);
            e.span.setAttribute(SemanticInternalAttributes.RETRY_DELAY, `${nextRetryDelay}ms`);
            e.span.end();
            await runtime.waitForDuration(nextRetryDelay);
            continue;
          } else if (e.originalError.name === "TypeError" && "cause" in e.originalError && e.originalError.cause instanceof Error) {
            const nextRetryDelay = calculateNextRetryDelay(resolveDefaults(init?.retry, "connectionError", defaultFetchRetryOptions.connectionError), attempt);
            if (!nextRetryDelay) {
              e.span.end();
              throw e;
            }
            if (attempt >= MAX_ATTEMPTS) {
              e.span.end();
              throw e;
            }
            e.span.setAttribute(SemanticInternalAttributes.RETRY_AT, new Date(Date.now() + nextRetryDelay).toISOString());
            e.span.setAttribute(SemanticInternalAttributes.RETRY_COUNT, attempt);
            e.span.setAttribute(SemanticInternalAttributes.RETRY_DELAY, `${nextRetryDelay}ms`);
            e.span.end();
            await runtime.waitForDuration(nextRetryDelay);
            continue;
          }
        }
        if (e instanceof FetchErrorWithSpan) {
          e.span.end();
        }
        throw e;
      } finally {
        attempt++;
      }
    }
  }, {
    attributes: {
      [SemanticInternalAttributes.STYLE_ICON]: "arrow-capsule",
      ...createFetchAttributes(input, init),
      ...createFetchRetryOptionsAttributes(init?.retry)
    }
  });
}
__name(retryFetch, "retryFetch");
var doFetchRequest = /* @__PURE__ */ __name(async (input, init, attemptCount = 0) => {
  const httpMethod = normalizeHttpMethod(input, init);
  const span = tracer.startSpan(`HTTP ${httpMethod}`, {
    attributes: {
      [SemanticInternalAttributes.STYLE_ICON]: "world",
      ...attemptCount > 1 ? {
        ["http.request.resend_count"]: attemptCount - 1
      } : {},
      ...createFetchAttributes(input, init)
    }
  });
  try {
    const response = await fetchWithInterceptors(input, {
      ...init,
      headers: {
        ...init?.headers,
        "x-retry-count": attemptCount.toString()
      }
    });
    span.setAttributes(createFetchResponseAttributes(response));
    if (!response.ok) {
      span.recordException(`${response.status}: ${response.statusText}`);
      span.setStatus({
        code: SpanStatusCode.ERROR,
        message: `${response.status}: ${response.statusText}`
      });
    }
    return [
      response,
      span
    ];
  } catch (e) {
    if (typeof e === "string" || e instanceof Error) {
      span.recordException(e);
    }
    span.setStatus({
      code: SpanStatusCode.ERROR
    });
    span.setAttribute(SEMATTRS_HTTP_STATUS_CODE, 0);
    span.setAttribute("http.status_text", "This operation was aborted.");
    throw new FetchErrorWithSpan(e, span);
  }
}, "doFetchRequest");
var calculateRetryDelayForResponse = /* @__PURE__ */ __name(async (retry2, response, attemptCount) => {
  if (!retry2) {
    return;
  }
  const strategy = await getRetryStrategyForResponse(response, retry2);
  if (!strategy) {
    return;
  }
  switch (strategy.strategy) {
    case "backoff": {
      const value = calculateNextRetryDelay({
        ...defaultRetryOptions,
        ...strategy
      }, attemptCount);
      if (value) {
        return {
          type: "delay",
          value
        };
      }
      break;
    }
    case "headers": {
      const resetAt = response.headers.get(strategy.resetHeader);
      if (typeof resetAt === "string") {
        const resetTimestamp = calculateResetAt(resetAt, strategy.resetFormat ?? "unix_timestamp_in_ms");
        if (resetTimestamp) {
          return {
            type: "timestamp",
            value: resetTimestamp
          };
        }
      }
      break;
    }
  }
}, "calculateRetryDelayForResponse");
var getRetryStrategyForResponse = /* @__PURE__ */ __name(async (response, retry2) => {
  const statusCodes = Object.keys(retry2);
  const clonedResponse = response.clone();
  for (let i = 0; i < statusCodes.length; i++) {
    const statusRange = statusCodes[i];
    const strategy = retry2[statusRange];
    if (isStatusCodeInRange(response.status, statusRange)) {
      if (strategy.bodyFilter) {
        const body = safeJsonParse(await clonedResponse.text());
        if (!body) {
          continue;
        }
        if (eventFilterMatches(body, strategy.bodyFilter)) {
          return strategy;
        } else {
          continue;
        }
      }
      return strategy;
    }
  }
}, "getRetryStrategyForResponse");
var isStatusCodeInRange = /* @__PURE__ */ __name((statusCode, statusRange) => {
  if (statusRange === "all") {
    return true;
  }
  if (statusRange.includes(",")) {
    const statusCodes = statusRange.split(",").map((s) => s.trim());
    return statusCodes.some((s) => isStatusCodeInRange(statusCode, s));
  }
  const [start, end] = statusRange.split("-");
  if (end) {
    return statusCode >= parseInt(start, 10) && statusCode <= parseInt(end, 10);
  }
  if (start.endsWith("xx")) {
    const prefix = start.slice(0, -2);
    const statusCodePrefix = Math.floor(statusCode / 100).toString();
    return statusCodePrefix === prefix;
  }
  const statusCodeString = statusCode.toString();
  const rangePrefix = start.slice(0, -1);
  if (start.endsWith("x") && statusCodeString.startsWith(rangePrefix)) {
    return true;
  }
  return statusCode === parseInt(start, 10);
}, "isStatusCodeInRange");
var createAttributesFromHeaders = /* @__PURE__ */ __name((headers) => {
  const attributes = {};
  const normalizedHeaderKey = /* @__PURE__ */ __name((key) => {
    return key.toLowerCase();
  }, "normalizedHeaderKey");
  headers.forEach((value, key) => {
    attributes[`http.response.header.${normalizedHeaderKey(key)}`] = value;
  });
  return attributes;
}, "createAttributesFromHeaders");
var safeJsonParse = /* @__PURE__ */ __name((json) => {
  try {
    return JSON.parse(json);
  } catch (e) {
    return null;
  }
}, "safeJsonParse");
var interceptFetch = /* @__PURE__ */ __name((...handlers) => {
  return {
    run: async (fn) => {
      const current = fetchHttpHandlerStorage.getStore();
      if (current) {
        current.push(...handlers);
        return fn();
      } else {
        return fetchHttpHandlerStorage.run(handlers, fn);
      }
    }
  };
}, "interceptFetch");
var resolveDefaults = /* @__PURE__ */ __name((obj, key, defaults) => {
  if (!obj) {
    return defaults;
  }
  if (obj[key] === void 0 || obj[key] === null) {
    return defaults;
  }
  return obj[key];
}, "resolveDefaults");
var createFetchAttributes = /* @__PURE__ */ __name((input, init) => {
  const url = normalizeUrlFromInput(input);
  const httpMethod = normalizeHttpMethod(input, init);
  return {
    [SEMATTRS_HTTP_METHOD]: httpMethod,
    [SEMATTRS_HTTP_URL]: url.href,
    [SEMATTRS_HTTP_HOST]: url.hostname,
    ["server.host"]: url.hostname,
    ["server.port"]: url.port,
    [SEMATTRS_HTTP_SCHEME]: url.protocol.replace(":", ""),
    ...accessoryAttributes({
      items: [
        {
          text: url.hostname,
          variant: "normal"
        }
      ],
      style: "codepath"
    })
  };
}, "createFetchAttributes");
var createFetchResponseAttributes = /* @__PURE__ */ __name((response) => {
  return {
    [SEMATTRS_HTTP_STATUS_CODE]: response.status,
    "http.status_text": response.statusText,
    [SEMATTRS_HTTP_RESPONSE_CONTENT_LENGTH]: response.headers.get("content-length") || "0",
    ...createAttributesFromHeaders(response.headers)
  };
}, "createFetchResponseAttributes");
var createFetchRetryOptionsAttributes = /* @__PURE__ */ __name((retry2) => {
  const byStatus = resolveDefaults(retry2, "byStatus", defaultFetchRetryOptions.byStatus);
  const connectionError = resolveDefaults(retry2, "connectionError", defaultFetchRetryOptions.connectionError);
  const timeout = resolveDefaults(retry2, "timeout", defaultFetchRetryOptions.timeout);
  return {
    ...flattenAttributes(byStatus, "retry.byStatus"),
    ...flattenAttributes(connectionError, "retry.connectionError"),
    ...flattenAttributes(timeout, "retry.timeout")
  };
}, "createFetchRetryOptionsAttributes");
var retry = {
  onThrow,
  fetch: retryFetch,
  interceptFetch
};
var runs = {
  replay: replayRun,
  cancel: cancelRun,
  retrieve: retrieveRun,
  list: listRuns,
  reschedule: rescheduleRun,
  poll
};
function listRuns(paramsOrProjectRef, paramsOrOptions, requestOptions) {
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  const $requestOptions = listRunsRequestOptions(paramsOrProjectRef, paramsOrOptions, requestOptions);
  if (typeof paramsOrProjectRef === "string") {
    if (isRequestOptions(paramsOrOptions)) {
      return apiClient.listProjectRuns(paramsOrProjectRef, {}, $requestOptions);
    } else {
      return apiClient.listProjectRuns(paramsOrProjectRef, paramsOrOptions, $requestOptions);
    }
  }
  return apiClient.listRuns(paramsOrProjectRef, $requestOptions);
}
__name(listRuns, "listRuns");
function listRunsRequestOptions(paramsOrProjectRef, paramsOrOptions, requestOptions) {
  if (typeof paramsOrProjectRef === "string") {
    if (isRequestOptions(paramsOrOptions)) {
      return mergeRequestOptions({
        tracer,
        name: "runs.list()",
        icon: "runs",
        attributes: {
          projectRef: paramsOrProjectRef,
          ...accessoryAttributes({
            items: [
              {
                text: paramsOrProjectRef,
                variant: "normal"
              }
            ],
            style: "codepath"
          })
        }
      }, paramsOrOptions);
    } else {
      return mergeRequestOptions({
        tracer,
        name: "runs.list()",
        icon: "runs",
        attributes: {
          projectRef: paramsOrProjectRef,
          ...flattenAttributes(paramsOrOptions, "queryParams"),
          ...accessoryAttributes({
            items: [
              {
                text: paramsOrProjectRef,
                variant: "normal"
              }
            ],
            style: "codepath"
          })
        }
      }, requestOptions);
    }
  }
  return mergeRequestOptions({
    tracer,
    name: "runs.list()",
    icon: "runs",
    attributes: {
      ...flattenAttributes(paramsOrProjectRef, "queryParams")
    }
  }, isRequestOptions(paramsOrOptions) ? paramsOrOptions : requestOptions);
}
__name(listRunsRequestOptions, "listRunsRequestOptions");
function retrieveRun(runId, requestOptions) {
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  const $requestOptions = mergeRequestOptions({
    tracer,
    name: "runs.retrieve()",
    icon: "runs",
    attributes: {
      runId: typeof runId === "string" ? runId : runId.id,
      ...accessoryAttributes({
        items: [
          {
            text: typeof runId === "string" ? runId : runId.id,
            variant: "normal"
          }
        ],
        style: "codepath"
      })
    }
  }, requestOptions);
  if (typeof runId === "string") {
    return apiClient.retrieveRun(runId, $requestOptions);
  } else {
    return apiClient.retrieveRun(runId.id, $requestOptions);
  }
}
__name(retrieveRun, "retrieveRun");
function replayRun(runId, requestOptions) {
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  const $requestOptions = mergeRequestOptions({
    tracer,
    name: "runs.replay()",
    icon: "runs",
    attributes: {
      runId,
      ...accessoryAttributes({
        items: [
          {
            text: runId,
            variant: "normal"
          }
        ],
        style: "codepath"
      })
    }
  }, requestOptions);
  return apiClient.replayRun(runId, $requestOptions);
}
__name(replayRun, "replayRun");
function cancelRun(runId, requestOptions) {
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  const $requestOptions = mergeRequestOptions({
    tracer,
    name: "runs.cancel()",
    icon: "runs",
    attributes: {
      runId,
      ...accessoryAttributes({
        items: [
          {
            text: runId,
            variant: "normal"
          }
        ],
        style: "codepath"
      })
    }
  }, requestOptions);
  return apiClient.cancelRun(runId, $requestOptions);
}
__name(cancelRun, "cancelRun");
function rescheduleRun(runId, body, requestOptions) {
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  const $requestOptions = mergeRequestOptions({
    tracer,
    name: "runs.reschedule()",
    icon: "runs",
    attributes: {
      runId,
      ...accessoryAttributes({
        items: [
          {
            text: runId,
            variant: "normal"
          }
        ],
        style: "codepath"
      })
    }
  }, requestOptions);
  return apiClient.rescheduleRun(runId, body, $requestOptions);
}
__name(rescheduleRun, "rescheduleRun");
var MAX_POLL_ATTEMPTS = 500;
async function poll(runId, options, requestOptions) {
  let attempts = 0;
  while (attempts++ < MAX_POLL_ATTEMPTS) {
    const run = await runs.retrieve(runId, requestOptions);
    if (run.isCompleted) {
      return run;
    }
    await new Promise((resolve) => setTimeout(resolve, options?.pollIntervalMs ?? 1e3));
  }
  throw new Error(`Run ${typeof runId === "string" ? runId : runId.id} did not complete after ${MAX_POLL_ATTEMPTS} attempts`);
}
__name(poll, "poll");
var idempotencyKeys = {
  create: createIdempotencyKey
};
function isIdempotencyKey(value) {
  return typeof value === "string" && value.length === 64;
}
__name(isIdempotencyKey, "isIdempotencyKey");
async function createIdempotencyKey(key, options) {
  const idempotencyKey = await generateIdempotencyKey([
    ...Array.isArray(key) ? key : [
      key
    ]
  ].concat(injectScope(options?.scope ?? "run")));
  return idempotencyKey;
}
__name(createIdempotencyKey, "createIdempotencyKey");
function injectScope(scope) {
  switch (scope) {
    case "run": {
      if (taskContext?.ctx) {
        return [
          taskContext.ctx.run.id
        ];
      }
    }
    case "attempt": {
      if (taskContext?.ctx) {
        return [
          taskContext.ctx.attempt.id
        ];
      }
    }
  }
  return [];
}
__name(injectScope, "injectScope");
async function generateIdempotencyKey(keyMaterial) {
  const hash = await crypto.subtle.digest("SHA-256", new TextEncoder().encode(keyMaterial.join("-")));
  return Array.from(new Uint8Array(hash)).map((byte) => byte.toString(16).padStart(2, "0")).join("");
}
__name(generateIdempotencyKey, "generateIdempotencyKey");

// src/v3/shared.ts
function queue(options) {
  return options;
}
__name(queue, "queue");
function createTask(params) {
  const customQueue = params.queue ? queue({
    name: params.queue?.name ?? `task/${params.id}`,
    ...params.queue
  }) : void 0;
  const task3 = {
    id: params.id,
    trigger: async (payload, options) => {
      const taskMetadata = taskCatalog.getTaskMetadata(params.id);
      return await trigger_internal(taskMetadata && taskMetadata.exportName ? `${taskMetadata.exportName}.trigger()` : `trigger()`, params.id, payload, {
        queue: customQueue,
        ...options
      });
    },
    batchTrigger: async (items) => {
      const taskMetadata = taskCatalog.getTaskMetadata(params.id);
      return await batchTrigger_internal(taskMetadata && taskMetadata.exportName ? `${taskMetadata.exportName}.batchTrigger()` : `batchTrigger()`, params.id, items, void 0, customQueue);
    },
    triggerAndWait: async (payload, options) => {
      const taskMetadata = taskCatalog.getTaskMetadata(params.id);
      return await triggerAndWait_internal(taskMetadata && taskMetadata.exportName ? `${taskMetadata.exportName}.triggerAndWait()` : `triggerAndWait()`, params.id, payload, {
        queue: customQueue,
        ...options
      });
    },
    batchTriggerAndWait: async (items) => {
      const taskMetadata = taskCatalog.getTaskMetadata(params.id);
      return await batchTriggerAndWait_internal(taskMetadata && taskMetadata.exportName ? `${taskMetadata.exportName}.batchTriggerAndWait()` : `batchTriggerAndWait()`, params.id, items, void 0, customQueue);
    }
  };
  taskCatalog.registerTaskMetadata({
    id: params.id,
    packageVersion: version,
    queue: params.queue,
    retry: params.retry ? {
      ...defaultRetryOptions,
      ...params.retry
    } : void 0,
    machine: params.machine,
    fns: {
      run: params.run,
      init: params.init,
      cleanup: params.cleanup,
      middleware: params.middleware,
      handleError: params.handleError,
      onSuccess: params.onSuccess,
      onFailure: params.onFailure,
      onStart: params.onStart
    }
  });
  return task3;
}
__name(createTask, "createTask");
async function trigger(id, payload, options, requestOptions) {
  return await trigger_internal("tasks.trigger()", id, payload, options, requestOptions);
}
__name(trigger, "trigger");
async function triggerAndWait(id, payload, options, requestOptions) {
  return await triggerAndWait_internal("tasks.triggerAndWait()", id, payload, options, requestOptions);
}
__name(triggerAndWait, "triggerAndWait");
async function batchTriggerAndWait(id, items, requestOptions) {
  return await batchTriggerAndWait_internal("tasks.batchTriggerAndWait()", id, items, requestOptions);
}
__name(batchTriggerAndWait, "batchTriggerAndWait");
async function triggerAndPoll(id, payload, options, requestOptions) {
  const handle = await trigger(id, payload, options, requestOptions);
  return runs.poll(handle, options, requestOptions);
}
__name(triggerAndPoll, "triggerAndPoll");
async function batchTrigger(id, items, requestOptions) {
  return await batchTrigger_internal("tasks.batchTrigger()", id, items, requestOptions);
}
__name(batchTrigger, "batchTrigger");
async function trigger_internal(name2, id, payload, options, requestOptions) {
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  const payloadPacket = await stringifyIO(payload);
  const handle = await apiClient.triggerTask(id, {
    payload: payloadPacket.data,
    options: {
      queue: options?.queue,
      concurrencyKey: options?.concurrencyKey,
      test: taskContext.ctx?.run.isTest,
      payloadType: payloadPacket.dataType,
      idempotencyKey: await makeKey(options?.idempotencyKey),
      delay: options?.delay,
      ttl: options?.ttl,
      tags: options?.tags,
      maxAttempts: options?.maxAttempts
    }
  }, {
    spanParentAsLink: true
  }, {
    name: name2,
    tracer,
    icon: "trigger",
    attributes: {
      [SEMATTRS_MESSAGING_OPERATION]: "publish",
      ["messaging.client_id"]: taskContext.worker?.id,
      [SEMATTRS_MESSAGING_SYSTEM]: "trigger.dev",
      ...accessoryAttributes({
        items: [
          {
            text: id,
            variant: "normal"
          }
        ],
        style: "codepath"
      })
    },
    onResponseBody: (body, span) => {
      body && typeof body === "object" && !Array.isArray(body) && "id" in body && typeof body.id === "string" && span.setAttribute("messaging.message.id", body.id);
    },
    ...requestOptions
  });
  return handle;
}
__name(trigger_internal, "trigger_internal");
async function batchTrigger_internal(name2, id, items, requestOptions, queue2) {
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  const response = await apiClient.batchTriggerTask(id, {
    items: await Promise.all(items.map(async (item) => {
      const payloadPacket = await stringifyIO(item.payload);
      return {
        payload: payloadPacket.data,
        options: {
          queue: item.options?.queue ?? queue2,
          concurrencyKey: item.options?.concurrencyKey,
          test: taskContext.ctx?.run.isTest,
          payloadType: payloadPacket.dataType,
          idempotencyKey: await makeKey(item.options?.idempotencyKey),
          delay: item.options?.delay,
          ttl: item.options?.ttl,
          tags: item.options?.tags,
          maxAttempts: item.options?.maxAttempts
        }
      };
    }))
  }, {
    spanParentAsLink: true
  }, {
    name: name2,
    tracer,
    icon: "trigger",
    attributes: {
      [SEMATTRS_MESSAGING_OPERATION]: "publish",
      ["messaging.client_id"]: taskContext.worker?.id,
      [SEMATTRS_MESSAGING_SYSTEM]: "trigger.dev",
      ...accessoryAttributes({
        items: [
          {
            text: id,
            variant: "normal"
          }
        ],
        style: "codepath"
      })
    },
    ...requestOptions
  });
  const handle = {
    batchId: response.batchId,
    runs: response.runs.map((id2) => ({
      id: id2
    }))
  };
  return handle;
}
__name(batchTrigger_internal, "batchTrigger_internal");
async function triggerAndWait_internal(name2, id, payload, options, requestOptions) {
  const ctx = taskContext.ctx;
  if (!ctx) {
    throw new Error("triggerAndWait can only be used from inside a task.run()");
  }
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  const payloadPacket = await stringifyIO(payload);
  return await tracer.startActiveSpan(name2, async (span) => {
    const response = await apiClient.triggerTask(id, {
      payload: payloadPacket.data,
      options: {
        dependentAttempt: ctx.attempt.id,
        lockToVersion: taskContext.worker?.version,
        queue: options?.queue,
        concurrencyKey: options?.concurrencyKey,
        test: taskContext.ctx?.run.isTest,
        payloadType: payloadPacket.dataType,
        idempotencyKey: await makeKey(options?.idempotencyKey),
        delay: options?.delay,
        ttl: options?.ttl,
        tags: options?.tags,
        maxAttempts: options?.maxAttempts
      }
    }, {}, requestOptions);
    span.setAttribute("messaging.message.id", response.id);
    if (options?.idempotencyKey) {
      const result2 = await apiClient.getRunResult(response.id);
      if (result2) {
        logger.log(`Result reused from previous task run with idempotency key '${options.idempotencyKey}'.`, {
          runId: response.id,
          idempotencyKey: options.idempotencyKey
        });
        return await handleTaskRunExecutionResult(result2);
      }
    }
    const result = await runtime.waitForTask({
      id: response.id,
      ctx
    });
    return await handleTaskRunExecutionResult(result);
  }, {
    kind: SpanKind.PRODUCER,
    attributes: {
      [SemanticInternalAttributes.STYLE_ICON]: "trigger",
      [SEMATTRS_MESSAGING_OPERATION]: "publish",
      ["messaging.client_id"]: taskContext.worker?.id,
      [SEMATTRS_MESSAGING_DESTINATION]: id,
      [SEMATTRS_MESSAGING_SYSTEM]: "trigger.dev",
      ...accessoryAttributes({
        items: [
          {
            text: id,
            variant: "normal"
          }
        ],
        style: "codepath"
      })
    }
  });
}
__name(triggerAndWait_internal, "triggerAndWait_internal");
async function batchTriggerAndWait_internal(name2, id, items, requestOptions, queue2) {
  const ctx = taskContext.ctx;
  if (!ctx) {
    throw new Error("batchTriggerAndWait can only be used from inside a task.run()");
  }
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  return await tracer.startActiveSpan(name2, async (span) => {
    const response = await apiClient.batchTriggerTask(id, {
      items: await Promise.all(items.map(async (item) => {
        const payloadPacket = await stringifyIO(item.payload);
        return {
          payload: payloadPacket.data,
          options: {
            lockToVersion: taskContext.worker?.version,
            queue: item.options?.queue ?? queue2,
            concurrencyKey: item.options?.concurrencyKey,
            test: taskContext.ctx?.run.isTest,
            payloadType: payloadPacket.dataType,
            idempotencyKey: await makeKey(item.options?.idempotencyKey),
            delay: item.options?.delay,
            ttl: item.options?.ttl,
            tags: item.options?.tags,
            maxAttempts: item.options?.maxAttempts
          }
        };
      })),
      dependentAttempt: ctx.attempt.id
    }, {}, requestOptions);
    span.setAttribute("messaging.message.id", response.batchId);
    const getBatchResults = /* @__PURE__ */ __name(async () => {
      const hasIdempotencyKey = items.some((item) => item.options?.idempotencyKey);
      if (hasIdempotencyKey) {
        const results = await apiClient.getBatchResults(response.batchId);
        if (results) {
          return results;
        }
      }
      return {
        id: response.batchId,
        items: []
      };
    }, "getBatchResults");
    const existingResults = await getBatchResults();
    const incompleteRuns = response.runs.filter((runId) => !existingResults.items.some((item) => item.id === runId));
    if (incompleteRuns.length === 0) {
      logger.log(`Results reused from previous task runs because of the provided idempotency keys.`);
      const runs3 = await handleBatchTaskRunExecutionResult(existingResults.items);
      return {
        id: existingResults.id,
        runs: runs3
      };
    }
    const result = await runtime.waitForBatch({
      id: response.batchId,
      runs: incompleteRuns,
      ctx
    });
    const combinedItems = [];
    for (const runId of response.runs) {
      const existingItem = existingResults.items.find((item) => item.id === runId);
      if (existingItem) {
        combinedItems.push(existingItem);
      } else {
        const newItem = result.items.find((item) => item.id === runId);
        if (newItem) {
          combinedItems.push(newItem);
        }
      }
    }
    const runs2 = await handleBatchTaskRunExecutionResult(combinedItems);
    return {
      id: result.id,
      runs: runs2
    };
  }, {
    kind: SpanKind.PRODUCER,
    attributes: {
      [SemanticInternalAttributes.STYLE_ICON]: "trigger",
      ["messaging.batch.message_count"]: items.length,
      [SEMATTRS_MESSAGING_OPERATION]: "publish",
      ["messaging.client_id"]: taskContext.worker?.id,
      [SEMATTRS_MESSAGING_DESTINATION]: id,
      [SEMATTRS_MESSAGING_SYSTEM]: "trigger.dev",
      ...accessoryAttributes({
        items: [
          {
            text: id,
            variant: "normal"
          }
        ],
        style: "codepath"
      })
    }
  });
}
__name(batchTriggerAndWait_internal, "batchTriggerAndWait_internal");
async function handleBatchTaskRunExecutionResult(items) {
  const someObjectStoreOutputs = items.some((item) => item.ok && item.outputType === "application/store");
  if (!someObjectStoreOutputs) {
    const results = await Promise.all(items.map(async (item) => {
      return await handleTaskRunExecutionResult(item);
    }));
    return results;
  }
  return await tracer.startActiveSpan("store.downloadPayloads", async (span) => {
    const results = await Promise.all(items.map(async (item) => {
      return await handleTaskRunExecutionResult(item);
    }));
    return results;
  }, {
    kind: SpanKind.INTERNAL,
    [SemanticInternalAttributes.STYLE_ICON]: "cloud-download"
  });
}
__name(handleBatchTaskRunExecutionResult, "handleBatchTaskRunExecutionResult");
async function handleTaskRunExecutionResult(execution) {
  if (execution.ok) {
    const outputPacket = {
      data: execution.output,
      dataType: execution.outputType
    };
    const importedPacket = await conditionallyImportPacket(outputPacket, tracer);
    return {
      ok: true,
      id: execution.id,
      output: await parsePacket(importedPacket)
    };
  } else {
    return {
      ok: false,
      id: execution.id,
      error: createErrorTaskError(execution.error)
    };
  }
}
__name(handleTaskRunExecutionResult, "handleTaskRunExecutionResult");
function apiClientMissingError() {
  const hasBaseUrl = !!apiClientManager.baseURL;
  const hasAccessToken = !!apiClientManager.accessToken;
  if (!hasBaseUrl && !hasAccessToken) {
    return `You need to set the TRIGGER_API_URL and TRIGGER_SECRET_KEY environment variables.`;
  } else if (!hasBaseUrl) {
    return `You need to set the TRIGGER_API_URL environment variable.`;
  } else if (!hasAccessToken) {
    return `You need to set the TRIGGER_SECRET_KEY environment variable.`;
  }
  return `Unknown error`;
}
__name(apiClientMissingError, "apiClientMissingError");
async function makeKey(idempotencyKey) {
  if (!idempotencyKey) {
    return;
  }
  if (isIdempotencyKey(idempotencyKey)) {
    return idempotencyKey;
  }
  return await idempotencyKeys.create(idempotencyKey, {
    scope: "global"
  });
}
__name(makeKey, "makeKey");

// src/v3/tasks.ts
function task(options) {
  return createTask(options);
}
__name(task, "task");
var tasks = {
  trigger,
  triggerAndPoll,
  batchTrigger,
  triggerAndWait,
  batchTriggerAndWait
};
var wait = {
  for: async (options) => {
    return tracer.startActiveSpan(`wait.for()`, async (span) => {
      const durationInMs = calculateDurationInMs(options);
      await runtime.waitForDuration(durationInMs);
    }, {
      attributes: {
        [SemanticInternalAttributes.STYLE_ICON]: "wait",
        ...accessoryAttributes({
          items: [
            {
              text: nameForWaitOptions(options),
              variant: "normal"
            }
          ],
          style: "codepath"
        })
      }
    });
  },
  until: async (options) => {
    return tracer.startActiveSpan(`wait.until()`, async (span) => {
      const start = Date.now();
      if (options.throwIfInThePast && options.date < /* @__PURE__ */ new Date()) {
        throw new Error("Date is in the past");
      }
      const durationInMs = options.date.getTime() - start;
      await runtime.waitForDuration(durationInMs);
    }, {
      attributes: {
        [SemanticInternalAttributes.STYLE_ICON]: "wait",
        ...accessoryAttributes({
          items: [
            {
              text: options.date.toISOString(),
              variant: "normal"
            }
          ],
          style: "codepath"
        })
      }
    });
  }
};
function nameForWaitOptions(options) {
  if ("seconds" in options) {
    return options.seconds === 1 ? `1 second` : `${options.seconds} seconds`;
  }
  if ("minutes" in options) {
    return options.minutes === 1 ? `1 minute` : `${options.minutes} minutes`;
  }
  if ("hours" in options) {
    return options.hours === 1 ? `1 hour` : `${options.hours} hours`;
  }
  if ("days" in options) {
    return options.days === 1 ? `1 day` : `${options.days} days`;
  }
  if ("weeks" in options) {
    return options.weeks === 1 ? `1 week` : `${options.weeks} weeks`;
  }
  if ("months" in options) {
    return options.months === 1 ? `1 month` : `${options.months} months`;
  }
  if ("years" in options) {
    return options.years === 1 ? `1 year` : `${options.years} years`;
  }
  return "NaN";
}
__name(nameForWaitOptions, "nameForWaitOptions");
function calculateDurationInMs(options) {
  if ("seconds" in options) {
    return options.seconds * 1e3;
  }
  if ("minutes" in options) {
    return options.minutes * 1e3 * 60;
  }
  if ("hours" in options) {
    return options.hours * 1e3 * 60 * 60;
  }
  if ("days" in options) {
    return options.days * 1e3 * 60 * 60 * 24;
  }
  if ("weeks" in options) {
    return options.weeks * 1e3 * 60 * 60 * 24 * 7;
  }
  if ("months" in options) {
    return options.months * 1e3 * 60 * 60 * 24 * 30;
  }
  if ("years" in options) {
    return options.years * 1e3 * 60 * 60 * 24 * 365;
  }
  throw new Error("Invalid options");
}
__name(calculateDurationInMs, "calculateDurationInMs");
var usage = {
  /**
  * Get the current running usage of this task run.
  *
  * @example
  *
  * ```typescript
  * import { usage, task } from "@systemfsoftware/trigger.dev_sdk/v3";
  *
  * export const myTask = task({
  *  id: "my-task",
  *  run: async (payload, { ctx }) => {
  *   // ... Do a bunch of work
  *
  *   const currentUsage = usage.getCurrent();
  *
  *   // You have access to the current compute cost and duration up to this point
  *   console.log("Current attempt compute cost and duration", {
  *     cost: currentUsage.compute.attempt.costInCents,
  *     duration: currentUsage.compute.attempt.durationMs,
  *   });
  *
  *   // You also can see the total compute cost and duration up to this point in the run, across all attempts
  *   console.log("Current total compute cost and duration", {
  *     cost: currentUsage.compute.total.costInCents,
  *     duration: currentUsage.compute.total.durationMs,
  *   });
  *
  *   // You can see the base cost of the run, which is the cost of the run before any compute costs
  *   console.log("Total cost", {
  *     cost: currentUsage.totalCostInCents,
  *     baseCost: currentUsage.baseCostInCents,
  *   });
  *  },
  * });
  * ```
  */
  getCurrent: () => {
    const sample = usage$1.sample();
    const machine = taskContext.ctx?.machine;
    const run = taskContext.ctx?.run;
    if (!sample) {
      return {
        compute: {
          attempt: {
            costInCents: 0,
            durationMs: 0
          },
          total: {
            costInCents: run?.costInCents ?? 0,
            durationMs: run?.durationMs ?? 0
          }
        },
        baseCostInCents: run?.baseCostInCents ?? 0,
        totalCostInCents: (run?.costInCents ?? 0) + (run?.baseCostInCents ?? 0)
      };
    }
    const currentCostInCents = machine?.centsPerMs ? sample.cpuTime * machine.centsPerMs : 0;
    return {
      compute: {
        attempt: {
          costInCents: currentCostInCents,
          durationMs: sample.cpuTime
        },
        total: {
          costInCents: (run?.costInCents ?? 0) + currentCostInCents,
          durationMs: (run?.durationMs ?? 0) + sample.cpuTime
        }
      },
      baseCostInCents: run?.baseCostInCents ?? 0,
      totalCostInCents: (run?.costInCents ?? 0) + currentCostInCents + (run?.baseCostInCents ?? 0)
    };
  },
  /**
  * Measure the cost and duration of a function.
  *
  * @example
  *
  * ```typescript
  * import { usage } from "@systemfsoftware/trigger.dev_sdk/v3";
  *
  * export const myTask = task({
  *  id: "my-task",
  *  run: async (payload, { ctx }) => {
  *    const { result, compute } = await usage.measure(async () => {
  *      // Do some work
  *      return "result";
  *    });
  *
  *    console.log("Result", result);
  *    console.log("Cost and duration", { cost: compute.costInCents, duration: compute.durationMs });
  *  },
  * });
  * ```
  */
  measure: async (cb) => {
    const measurement = usage$1.start();
    const result = await cb();
    const sample = usage$1.stop(measurement);
    const machine = taskContext.ctx?.machine;
    const costInCents = machine?.centsPerMs ? sample.cpuTime * machine.centsPerMs : 0;
    return {
      result,
      compute: {
        costInCents,
        durationMs: sample.cpuTime
      }
    };
  }
};
var tags = {
  add: addTags
};
async function addTags(tags2, requestOptions) {
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  const run = taskContext.ctx?.run;
  if (!run) {
    throw new Error("Can't set tags outside of a run. You can trigger a task and set tags in the options.");
  }
  const $requestOptions = mergeRequestOptions({
    tracer,
    name: "tags.set()",
    icon: "tag",
    attributes: {
      ...accessoryAttributes({
        items: [
          {
            text: typeof tags2 === "string" ? tags2 : tags2.join(", "),
            variant: "normal"
          }
        ],
        style: "codepath"
      })
    }
  }, requestOptions);
  try {
    await apiClient.addTags(run.id, {
      tags: tags2
    }, $requestOptions);
  } catch (error) {
    if (error instanceof UnprocessableEntityError) {
      logger.error(error.message, {
        existingTags: run.tags,
        newTags: tags2
      });
      return;
    }
    logger.error("Failed to set tags", {
      error
    });
    throw error;
  }
}
__name(addTags, "addTags");

// src/v3/schedules/index.ts
var schedules_exports = {};
__export(schedules_exports, {
  activate: () => activate,
  create: () => create,
  deactivate: () => deactivate,
  del: () => del,
  list: () => list,
  retrieve: () => retrieve,
  task: () => task2,
  timezones: () => timezones,
  update: () => update
});
function task2(params) {
  const task3 = createTask(params);
  const cron = params.cron ? typeof params.cron === "string" ? params.cron : params.cron.pattern : void 0;
  const timezone = (params.cron && typeof params.cron !== "string" ? params.cron.timezone : "UTC") ?? "UTC";
  taskCatalog.updateTaskMetadata(task3.id, {
    triggerSource: "schedule",
    schedule: cron ? {
      cron,
      timezone
    } : void 0
  });
  return task3;
}
__name(task2, "task");
function create(options, requestOptions) {
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  const $requestOptions = mergeRequestOptions({
    tracer,
    name: "schedules.create()",
    icon: "clock",
    attributes: {
      ...accessoryAttributes({
        items: [
          {
            text: options.cron,
            variant: "normal"
          }
        ],
        style: "codepath"
      })
    }
  }, requestOptions);
  return apiClient.createSchedule(options, $requestOptions);
}
__name(create, "create");
function retrieve(scheduleId, requestOptions) {
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  const $requestOptions = mergeRequestOptions({
    tracer,
    name: "schedules.retrieve()",
    icon: "clock",
    attributes: {
      scheduleId,
      ...accessoryAttributes({
        items: [
          {
            text: scheduleId,
            variant: "normal"
          }
        ],
        style: "codepath"
      })
    }
  }, requestOptions);
  return apiClient.retrieveSchedule(scheduleId, $requestOptions);
}
__name(retrieve, "retrieve");
function update(scheduleId, options, requestOptions) {
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  const $requestOptions = mergeRequestOptions({
    tracer,
    name: "schedules.update()",
    icon: "clock",
    attributes: {
      scheduleId,
      ...accessoryAttributes({
        items: [
          {
            text: scheduleId,
            variant: "normal"
          }
        ],
        style: "codepath"
      })
    }
  }, requestOptions);
  return apiClient.updateSchedule(scheduleId, options, $requestOptions);
}
__name(update, "update");
function del(scheduleId, requestOptions) {
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  const $requestOptions = mergeRequestOptions({
    tracer,
    name: "schedules.delete()",
    icon: "clock",
    attributes: {
      scheduleId,
      ...accessoryAttributes({
        items: [
          {
            text: scheduleId,
            variant: "normal"
          }
        ],
        style: "codepath"
      })
    }
  }, requestOptions);
  return apiClient.deleteSchedule(scheduleId, $requestOptions);
}
__name(del, "del");
function deactivate(scheduleId, requestOptions) {
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  const $requestOptions = mergeRequestOptions({
    tracer,
    name: "schedules.deactivate()",
    icon: "clock",
    attributes: {
      scheduleId,
      ...accessoryAttributes({
        items: [
          {
            text: scheduleId,
            variant: "normal"
          }
        ],
        style: "codepath"
      })
    }
  }, requestOptions);
  return apiClient.deactivateSchedule(scheduleId, $requestOptions);
}
__name(deactivate, "deactivate");
function activate(scheduleId, requestOptions) {
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  const $requestOptions = mergeRequestOptions({
    tracer,
    name: "schedules.activate()",
    icon: "clock",
    attributes: {
      scheduleId,
      ...accessoryAttributes({
        items: [
          {
            text: scheduleId,
            variant: "normal"
          }
        ],
        style: "codepath"
      })
    }
  }, requestOptions);
  return apiClient.activateSchedule(scheduleId, $requestOptions);
}
__name(activate, "activate");
function list(options, requestOptions) {
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  const $requestOptions = mergeRequestOptions({
    tracer,
    name: "schedules.list()",
    icon: "clock"
  }, requestOptions);
  return apiClient.listSchedules(options, $requestOptions);
}
__name(list, "list");
function timezones(options) {
  const baseUrl = apiClientManager.baseURL;
  if (!baseUrl) {
    throw apiClientMissingError();
  }
  return zodfetch(TimezonesResult, `${baseUrl}/api/v1/timezones${options?.excludeUtc === true ? "?excludeUtc=true" : ""}`, {
    method: "GET",
    headers: {
      "Content-Type": "application/json"
    }
  });
}
__name(timezones, "timezones");

// src/v3/envvars.ts
var envvars_exports = {};
__export(envvars_exports, {
  create: () => create2,
  del: () => del2,
  list: () => list2,
  retrieve: () => retrieve2,
  update: () => update2,
  upload: () => upload
});
function upload(projectRefOrParams, slugOrRequestOptions, params, requestOptions) {
  let $projectRef;
  let $params;
  let $slug;
  const $requestOptions = overloadRequestOptions("upload", slugOrRequestOptions, requestOptions);
  if (taskContext.ctx) {
    if (typeof projectRefOrParams === "string") {
      $projectRef = projectRefOrParams;
      $slug = typeof slugOrRequestOptions === "string" ? slugOrRequestOptions : taskContext.ctx.environment.slug;
      if (!params) {
        throw new Error("params is required");
      }
      $params = params;
    } else {
      $params = projectRefOrParams;
      $projectRef = taskContext.ctx.project.ref;
      $slug = taskContext.ctx.environment.slug;
    }
  } else {
    if (typeof projectRefOrParams !== "string") {
      throw new Error("projectRef is required");
    }
    if (!slugOrRequestOptions || typeof slugOrRequestOptions !== "string") {
      throw new Error("slug is required");
    }
    if (!params) {
      throw new Error("params is required");
    }
    $projectRef = projectRefOrParams;
    $slug = slugOrRequestOptions;
    $params = params;
  }
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  return apiClient.importEnvVars($projectRef, $slug, $params, $requestOptions);
}
__name(upload, "upload");
function list2(projectRefOrRequestOptions, slug, requestOptions) {
  const $projectRef = !isRequestOptions(projectRefOrRequestOptions) ? projectRefOrRequestOptions : taskContext.ctx?.project.ref;
  const $slug = slug ?? taskContext.ctx?.environment.slug;
  let $requestOptions = isRequestOptions(projectRefOrRequestOptions) ? projectRefOrRequestOptions : requestOptions;
  if (!$projectRef) {
    throw new Error("projectRef is required");
  }
  if (!$slug) {
    throw new Error("slug is required");
  }
  $requestOptions = mergeRequestOptions({
    tracer,
    name: "envvars.list()",
    icon: "id-badge"
  }, $requestOptions);
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  return apiClient.listEnvVars($projectRef, $slug, $requestOptions);
}
__name(list2, "list");
function create2(projectRefOrParams, slugOrRequestOptions, params, requestOptions) {
  let $projectRef;
  let $slug;
  let $params;
  const $requestOptions = overloadRequestOptions("create", slugOrRequestOptions, requestOptions);
  if (taskContext.ctx) {
    if (typeof projectRefOrParams === "string") {
      $projectRef = projectRefOrParams;
      $slug = typeof slugOrRequestOptions === "string" ? slugOrRequestOptions : taskContext.ctx.environment.slug;
      if (!params) {
        throw new Error("params is required");
      }
      $params = params;
    } else {
      $params = projectRefOrParams;
      $projectRef = taskContext.ctx.project.ref;
      $slug = taskContext.ctx.environment.slug;
    }
  } else {
    if (typeof projectRefOrParams !== "string") {
      throw new Error("projectRef is required");
    }
    if (!slugOrRequestOptions || typeof slugOrRequestOptions !== "string") {
      throw new Error("slug is required");
    }
    if (!params) {
      throw new Error("params is required");
    }
    $projectRef = projectRefOrParams;
    $slug = slugOrRequestOptions;
    $params = params;
  }
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  return apiClient.createEnvVar($projectRef, $slug, $params, $requestOptions);
}
__name(create2, "create");
function retrieve2(projectRefOrName, slugOrRequestOptions, name1, requestOptions) {
  let $projectRef;
  let $slug;
  let $name;
  const $requestOptions = overloadRequestOptions("retrieve", slugOrRequestOptions, requestOptions);
  if (typeof name1 === "string") {
    $projectRef = projectRefOrName;
    $slug = typeof slugOrRequestOptions === "string" ? slugOrRequestOptions : taskContext.ctx?.environment.slug;
    $name = name1;
  } else {
    $projectRef = taskContext.ctx?.project.ref;
    $slug = taskContext.ctx?.environment.slug;
    $name = projectRefOrName;
  }
  if (!$projectRef) {
    throw new Error("projectRef is required");
  }
  if (!$slug) {
    throw new Error("slug is required");
  }
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  return apiClient.retrieveEnvVar($projectRef, $slug, $name, $requestOptions);
}
__name(retrieve2, "retrieve");
function del2(projectRefOrName, slugOrRequestOptions, name1, requestOptions) {
  let $projectRef;
  let $slug;
  let $name;
  const $requestOptions = overloadRequestOptions("del", slugOrRequestOptions, requestOptions);
  if (typeof name1 === "string") {
    $projectRef = projectRefOrName;
    $slug = typeof slugOrRequestOptions === "string" ? slugOrRequestOptions : taskContext.ctx?.environment.slug;
    $name = name1;
  } else {
    $projectRef = taskContext.ctx?.project.ref;
    $slug = taskContext.ctx?.environment.slug;
    $name = projectRefOrName;
  }
  if (!$projectRef) {
    throw new Error("projectRef is required");
  }
  if (!$slug) {
    throw new Error("slug is required");
  }
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  return apiClient.deleteEnvVar($projectRef, $slug, $name, $requestOptions);
}
__name(del2, "del");
function update2(projectRefOrName, slugOrParams, nameOrRequestOptions, params, requestOptions) {
  let $projectRef;
  let $slug;
  let $name;
  let $params;
  const $requestOptions = overloadRequestOptions("update", nameOrRequestOptions, requestOptions);
  if (taskContext.ctx) {
    if (typeof slugOrParams === "string") {
      $projectRef = slugOrParams;
      $slug = slugOrParams ?? taskContext.ctx.environment.slug;
      $name = typeof nameOrRequestOptions === "string" ? nameOrRequestOptions : taskContext.ctx.environment.slug;
      if (!params) {
        throw new Error("params is required");
      }
      $params = params;
    } else {
      $params = slugOrParams;
      $projectRef = taskContext.ctx.project.ref;
      $slug = taskContext.ctx.environment.slug;
      $name = projectRefOrName;
    }
  } else {
    if (typeof slugOrParams !== "string") {
      throw new Error("slug is required");
    }
    if (!projectRefOrName) {
      throw new Error("projectRef is required");
    }
    if (!params) {
      throw new Error("params is required");
    }
    $projectRef = projectRefOrName;
    $slug = slugOrParams;
    $name = name;
    $params = params;
  }
  const apiClient = apiClientManager.client;
  if (!apiClient) {
    throw apiClientMissingError();
  }
  return apiClient.updateEnvVar($projectRef, $slug, $name, $params, $requestOptions);
}
__name(update2, "update");
function overloadRequestOptions(name1, slugOrRequestOptions, requestOptions) {
  if (isRequestOptions(slugOrRequestOptions)) {
    return mergeRequestOptions({
      tracer,
      name: `envvars.${name1}()`,
      icon: "id-badge"
    }, slugOrRequestOptions);
  } else {
    return mergeRequestOptions({
      tracer,
      name: `envvars.${name1}()`,
      icon: "id-badge"
    }, requestOptions);
  }
}
__name(overloadRequestOptions, "overloadRequestOptions");

// src/v3/index.ts
function configure(options) {
  apiClientManager.setGlobalAPIClientConfiguration(options);
}
__name(configure, "configure");

export { InMemoryCache, configure, createCache, envvars_exports as envvars, idempotencyKeys, isIdempotencyKey, queue, retry, runs, schedules_exports as schedules, tags, task, tasks, usage, wait };
//# sourceMappingURL=out.js.map
//# sourceMappingURL=index.mjs.map